searchNodes=[{"doc":"ExRay defines an annotation construct that wraps regular functions and enable them to be traced using a simple affordance @trace to interact with an OpenTracing compliant collector. OpenTracing defines the concept of spans that track the call stack and record timing information and various call artifacts that can be used for application runtime inspection. This is a really cool piece of technology that compliments your monitoring solution as you now have x-ray vision of your application at runtime once a monitoring metric gets off the chart. However, in practice, your code gets quickly cluttered by your tracing efforts. ExRay alleviates the clutter by injecting cross-cutting tracing concern into your application code. By using @trace annotation, you can trap the essence of the calls without introducing tracing code mixed-in with your business logic. ExRay leverages Otter Erlang OpenTracing lib from the fine folks of BlueHouse Technology. To enable OpenTracing with ExRay defmodule Traceme do use ExRay , pre : :pre_fun , post : :post_fun alias ExRay.Span @trace kind : :cool_kid def elvis ( a , b ) , do : a + b defp pre_fun ( ctx ) do ctx . target |&gt; Span . open ( &quot;123&quot; ) # where 123 represent a unique callstack ID |&gt; :otter . tag ( :kind , ctx . meta [ :kind ] ) |&gt; :otter . log ( &quot;Calling Elvis!&quot; ) end defp post_fun ( _ctx , span , _res ) do span |&gt; :otter . log ( &quot;Elvis has left the building!&quot; ) |&gt; Span . close ( &quot;123&quot; ) end end ExRay provisions an ExRay.Context with function details and metadata that comes from the annotation. You can leverage this information for tracing in your pre and post span-hook functions.","ref":"ExRay.html","title":"ExRay","type":"module"},{"doc":"Defines a @trace annotation to enable regular function to be traced. Calling an annotated function f1 will ensure that a new span is created upon invocation and closed when the function exits. This macro will define a new function that will call the original function but decorated with pre and post tracing hooks. Tracing functions are generated at compile time.","ref":"ExRay.html#__using__/1","title":"ExRay.__using__/1","type":"macro"},{"doc":"Generates a new tracing function with before and after hooks that will call the original function. Pre and Post hooks can be defined at the module level or overriden on a per function basis.","ref":"ExRay.html#before_compile/1","title":"ExRay.before_compile/1","type":"macro"},{"doc":"Flags functions that have been annotated for further processing during the compilation phase","ref":"ExRay.html#on_definition/6","title":"ExRay.on_definition/6","type":"function"},{"doc":"Captures the essence of decorated function call. It contains the following elements: target: The name of the annotated function args: A collection of arguments passed to the wrapped function guards: A collection of guard clauses that identifies the function meta: Metadata specified in the annotation","ref":"ExRay.Context.html","title":"ExRay.Context","type":"module"},{"doc":"","ref":"ExRay.Context.html#t:t/0","title":"ExRay.Context.t/0","type":"type"},{"doc":"A set of convenience functions to manage spans.","ref":"ExRay.Span.html","title":"ExRay.Span","type":"module"},{"doc":"Closes the given span and pops the span state in the associated ETS span chain.","ref":"ExRay.Span.html#close/2","title":"ExRay.Span.close/2","type":"function"},{"doc":"Create a new root span with a given name and unique request chain ID. The request ID uniquely identifies the call chain and will be used as the primary key in the ETS table tracking the span chain.","ref":"ExRay.Span.html#open/2","title":"ExRay.Span.open/2","type":"function"},{"doc":"Creates a new span with a given parent span","ref":"ExRay.Span.html#open/3","title":"ExRay.Span.open/3","type":"function"},{"doc":"Convenience to retrive the parent span ID from a given span","ref":"ExRay.Span.html#parent_id/1","title":"ExRay.Span.parent_id/1","type":"function"},{"doc":"Store the span chains in an ETS table. The table must be created during the application initialization using the create call. The span chain acts like a call stack by pushing and popping spans as they come in and out of scope.","ref":"ExRay.Store.html","title":"ExRay.Store","type":"module"},{"doc":"Initializes the spans ETS table. The span table can be shared across process boundary.","ref":"ExRay.Store.html#create/0","title":"ExRay.Store.create/0","type":"function"},{"doc":"Fetch the top level span for a given key","ref":"ExRay.Store.html#current/1","title":"ExRay.Store.current/1","type":"function"},{"doc":"Fetch span stack for the given key","ref":"ExRay.Store.html#get/1","title":"ExRay.Store.get/1","type":"function"},{"doc":"Pops the top span off the stack.","ref":"ExRay.Store.html#pop/1","title":"ExRay.Store.pop/1","type":"function"},{"doc":"Pushes a new span to the span stack. The key must be unique.","ref":"ExRay.Store.html#push/2","title":"ExRay.Store.push/2","type":"function"},{"doc":"","ref":"ExRayAssist.html","title":"ExRayAssist","type":"module"},{"doc":"ExRay","ref":"readme.html","title":"ExRay","type":"extras"},{"doc":"ExRay defines an annotation construct that wraps regular functions and enable them to be traced using a simple affordance @trace to interact with an OpenTracing compliant collector. OpenTracing defines the concept of spans that track the call stack and record timing information and various call artifacts that can be used for application runtime inspection. This is a really cool piece of technology that compliments your monitoring solution as you now have x-ray vision of your application at runtime once a monitoring metric gets off the chart. However in practice, your code gets quickly cluttered by your tracing efforts. ExRay alleviates the clutter by injecting cross-cutting tracing concern into your application code. By using @trace annotation, you can trap the essence of the calls without introducing tracing code mixed-in with your business logic. ExRay leverages Otter Erlang OpenTracing lib from the fine folks of BlueHouse Technology.","ref":"readme.html#motivation","title":"ExRay - Motivation","type":"extras"},{"doc":"ExRay","ref":"readme.html#documentation","title":"ExRay - Documentation","type":"extras"},{"doc":"Tracing information needs to be collected by a tracing backend of your choice. You can run ExRay using any trace collector that Otter supports. In the following example we will use Jaeger from the self-driven folks at Uber! Start Jaeger Providing you have docker running on you box, you can use the all-in-one Jaeger image to get you started. If you are a Kubernetes fan, you can use the Jaeger chart docker run -d -e COLLECTOR_ZIPKIN_HTTP_PORT=9411 -p5775:5775/udp -p6831:6831/udp -p6832:6832/udp \\ -p5778:5778 -p16686:16686 -p14268:14268 -p9411:9411 jaegertracing/all-in-one:latest Alternatively, you could use the echo server otter_srv for testing your installation but I find it more fun to have a cool UI to look at your tracing outcomes. Setup Dependencies Add the following dependencies to your project (Elixir or Phoenix) NOTE: You can use Httpc(default), Hackney or IBrowse for your http client. We opt for Ibrowse here. def deps do [ ... { :ex_ray , &quot;~&gt; 0.1.2&quot; } , { :ibrowse , &quot;~&gt; 4.4.0&quot; } ] end Configure Otter In your config file, you need to tell Otter where to find your Zipkin collector. config :otter , zipkin_collector_uri : &#39;http://127.0.0.1:9411/api/v1/spans&#39; , zipkin_tag_host_service : &quot;TraceMe&quot; , http_client : :ibrowse Note: This thru me for a loop! The uri is indeed a char list and not a string! Configure Your Application ExRay uses ETS to track the span chains. Each request will create a new chain that will grow and collapse with your function call stack. As such you will need to track a unique call ID either by generating a custom ID for each request. If you are using Phoenix, the framework does this for you by using the request_id in the response headers. The ETS table is used to locate the parent span for which to attach to when navigating the call stack. NOTE: One cool aspect of OpenTracing is the tracing does not stop at the process boundaries as you can continue the chain to other processes and external services. Please see the examples directory for further info. In your application initialization, you need to make sure the ExRay ETS table is created by calling # application.ex ... ExRay.Store . create ( ) ... Let's Trace! Here is a simples tracing example. Please take a look at the examples in the Repo and ExRay Tracers for Phoenix sample use cases. defmodule TraceMe do use ExRay , pre : :before_fun , post : :after_fun alias ExRay.Span # Generates a request id @req_id :os . system_time ( :milli_seconds ) |&gt; Integer . to_string |&gt; IO . inspect @trace kind : :critical def fred ( a , b ) , do : a + b defp before_fun ( ctx ) do ctx . target |&gt; Span . open ( @req_id ) |&gt; :otter . tag ( :kind , ctx . meta [ :kind ] ) |&gt; :otter . log ( &quot;&gt;&gt;&gt; \#{ ctx . target } with \#{ ctx . args |&gt; inspect } &quot; ) end defp after_fun ( ctx , span , res ) do span |&gt; :otter . log ( &quot;&lt;&lt;&lt; \#{ ctx . target } returned \#{ res } &quot; ) |&gt; Span . close ( @req_id ) end end See it! That's great, but how do you see the tracing information? open http://localhost:16686 Under Service open up our TraceMe service, click Find Traces and you should now see your span in all its glory ie timing, tag and log info Thank you for playing! © 2017 Imhotep Software LLC. All materials licensed under Apache v2.0","ref":"readme.html#installation","title":"ExRay - Installation","type":"extras"}]